<!DOCTYPE html>
<html lang="en">
<head>

  <!-- Basic Page Needs
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <meta charset="utf-8">
  <title>{{title}}</title>
  <meta name="description" content="">
  <meta name="author" content="">

  <!-- Mobile Specific Metas
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <meta name="viewport" content="width=device-width, initial-scale=1">

  {# DEBUG: Reming of next two rows allows to debug current code w/o live internet (don't need to make external connections). #}
  <!-- FONT
    –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link href="//fonts.googleapis.com/css?family=Raleway:400,300,600" rel="stylesheet" type="text/css">
  <!-- ICONS
    –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.4.0/css/all.min.css">

  <!-- CSS
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link rel="stylesheet" href="{{ url_for('static', filename='css/normalize.css') }}">
  <link rel="stylesheet" href="{{ url_for('static', filename='css/tooltip.css') }}">
  <link rel="stylesheet" href="{{ url_for('static', filename='css/skeleton.css') }}">
  <link rel="stylesheet" href="{{ url_for('static', filename='css/customizations.css') }}">
  <link rel="stylesheet" href="{{ url_for('static', filename='css/common.css') }}">
  <link rel="stylesheet" href="{{ url_for('static', filename='css/fixed-table.css') }}">

  <!-- Scripts
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <script src="{{ url_for('static', filename='js/libs/ym/ym-modules-0.1.2.min.js') }}"></script>
  <script src="{{ url_for('static', filename='js/common/CommonHelpers.js') }}"></script>
  {# TODO: to move to the body end? #}

  <!-- Favicon
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link rel="icon" type="image/png" href="{{ url_for('static', filename='images/favicon.ico') }}">

</head>
<body>
